@@ -1,5 +1,6 @@
 from AOT_biomaps.Config import config
 
+from scipy.signal import hilbert
 import os
 import h5py
 import numpy as np
@@ -109,136 +110,55 @@ def reshape_field(field, factor, device=None):
     if isinstance(field, np.ndarray):
         return downsampled.cpu().numpy()
     else:
-        return downsampled
+        return downsampled.cpu().numpy()
 
-
-def CPU_hilbert(signal, axis=0):
+def calculate_envelope_squared(field):
     """
-    Compute the Hilbert transform of a real signal using NumPy.
+    Calcule l'enveloppe au carré du champ acoustique en utilisant scipy.signal.hilbert (CPU uniquement).
 
-    Parameters:
-    - signal: Input real signal (numpy.ndarray).
-    - axis: Axis along which to compute the Hilbert transform.
+    Args:
+        field: Champ acoustique (numpy.ndarray) de forme (T, X, Z) ou (T, X, Y, Z).
 
     Returns:
-    - analytic_signal: The analytic signal of the input.
-    """
-    fft_signal = np.fft.fftn(signal, axes=[axis])
-    h = np.zeros_like(signal)
-
-    if axis == 0:
-        h[0 : signal.shape[0] // 2 + 1, ...] = 1
-        h[signal.shape[0] // 2 + 1 :, ...] = 2
-    else:
-        raise ValueError("Axis not supported for this implementation.")
-
-    analytic_signal = np.fft.ifftn(fft_signal * h, axes=[axis])
-    return analytic_signal
-
-def GPU_hilbert(signal, axis=0):
-    """
-    Compute the Hilbert transform of a real signal using PyTorch.
-
-    Parameters:
-    - signal: Input real signal (torch.Tensor).
-    - axis: Axis along which to compute the Hilbert transform.
-
-    Returns:
-    - analytic_signal: The analytic signal of the input.
-    """
-    fft_signal = torch.fft.fftn(signal, dim=axis)
-    h = torch.zeros_like(signal)
-    if axis == 0:
-        h[0 : signal.shape[0] // 2 + 1, ...] = 1
-        h[signal.shape[0] // 2 + 1 :, ...] = 2
-    else:
-        raise ValueError("Axis not supported for this implementation.")
-
-    analytic_signal = torch.fft.ifftn(fft_signal * h, dim=axis)
-    return analytic_signal
-
-def calculate_envelope_squared(field, isGPU):
-    """
-    Calculate the analytic envelope of the acoustic field using either CPU or GPU with PyTorch.
-    Parameters:
-        - field: Input acoustic field (numpy.ndarray or torch.Tensor).
-        - isGPU (bool): If True, use GPU for computation. Otherwise, use CPU.
-    Returns:
-        - envelope (numpy.ndarray): The squared analytic envelope of the acoustic field.
+        envelope (numpy.ndarray): Enveloppe au carré du champ acoustique.
     """
     try:
         if field is None:
-            raise ValueError("Acoustic field is not generated. Please generate the field first.")
+            raise ValueError("Le champ acoustique n'est pas généré. Veuillez d'abord générer le champ.")
 
-        # Convert input to tensor (handle both numpy arrays and tensors)
-        if isinstance(field, np.ndarray):
-            acoustic_field = torch.from_numpy(field).to(dtype=torch.float32)
-        else:
-            acoustic_field = field.detach().clone().to(dtype=torch.float32)
-
-        # Handle GPU/CPU transfer
-        if isGPU:
-            if not torch.cuda.is_available():
-                print("CUDA is not available, falling back to CPU.")
-                isGPU = False
+        if not isinstance(field, np.ndarray):
+            if hasattr(field, 'cpu'):
+                field = field.cpu().numpy()  # Si c'est un tenseur PyTorch sur GPU/CPU
             else:
-                # Check GPU memory
-                free_memory = torch.cuda.get_device_properties(0).total_memory - torch.cuda.memory_allocated(0)
-                required_memory = acoustic_field.numel() * acoustic_field.element_size()
-                if free_memory < required_memory:
-                    print(f"GPU memory insufficient ({required_memory / (1024 ** 2):.2f} MB required, {free_memory / (1024 ** 2):.2f} MB free), falling back to CPU.")
-                    isGPU = False
-                else:
-                    acoustic_field = acoustic_field.cuda()
+                field = np.array(field)  # Conversion générique
 
-        if len(acoustic_field.shape) not in [3, 4]:
-            raise ValueError("Input acoustic field must be a 3D or 4D array.")
+        if len(field.shape) not in [3, 4]:
+            raise ValueError("Le champ acoustique doit être un tableau 3D (T, X, Z) ou 4D (T, X, Y, Z).")
 
-        def process_slice(slice_index):
-            """Calculate the envelope for a given slice of the acoustic field."""
-            slice_data = acoustic_field[slice_index]
+        # Calcul de l'enveloppe avec scipy.signal.hilbert
+        if len(field.shape) == 3:
+            T, X, Z = field.shape
+            envelope = np.zeros_like(field)
+            for x in range(X):
+                for z in range(Z):
+                    envelope[:, x, z] = np.abs(hilbert(field[:, x, z], axis=0)) ** 2
+        elif len(field.shape) == 4:
+            T, X, Y, Z = field.shape
+            envelope = np.zeros_like(field)
+            for x in range(X):
+                for y in range(Y):
+                    for z in range(Z):
+                        envelope[:, x, y, z] = np.abs(hilbert(field[:, x, y, z], axis=0)) ** 2
+                        
 
-            if len(acoustic_field.shape) == 3:
-                if isGPU:
-                    return torch.abs(GPU_hilbert(slice_data, axis=0))**2
-                else:
-                    return torch.from_numpy(np.abs(CPU_hilbert(slice_data.cpu().numpy(), axis=0))**2).to(dtype=torch.float32)
-
-            elif len(acoustic_field.shape) == 4:
-                if isGPU:
-                    return torch.stack([
-                        torch.abs(GPU_hilbert(slice_data[:, y, z], axis=0))**2
-                        for y in range(slice_data.shape[1])
-                        for z in range(slice_data.shape[2])
-                    ]).reshape(slice_data.shape[1], slice_data.shape[2], -1).permute(2, 0, 1)
-                else:
-                    envelope = torch.zeros_like(slice_data)
-                    for y in range(slice_data.shape[1]):
-                        for z in range(slice_data.shape[2]):
-                            envelope[:, y, z] = torch.from_numpy(
-                                np.abs(CPU_hilbert(slice_data[:, y, z].cpu().numpy(), axis=0))**2
-                            )
-                    return envelope
-
-        # Process slices
-        num_slices = acoustic_field.shape[0]
-        slice_indices = range(num_slices)
-
-        if isGPU:
-            envelopes = [process_slice(i) for i in slice_indices]
-        else:
-            with ThreadPoolExecutor() as executor:
-                envelopes = list(executor.map(process_slice, slice_indices))
-
-        # Combine results
-        envelope = torch.stack(envelopes, axis=0)
-        return envelope.cpu().numpy() if isGPU else envelope.numpy()
+        return envelope
 
     except Exception as e:
-        print(f"Error in calculate_envelope_squared method: {e}")
+        print(f"Erreur dans calculate_envelope_squared: {e}")
         raise
 
 
+
 def getPattern(pathFile):
     """
     Get the pattern from a file path.
