@@ -1,6 +1,6 @@
 import AOT_biomaps.Settings
 from AOT_biomaps.Config import config
-from AOT_biomaps.AOT_Acoustic.AcousticTools import calculate_envelope_squared, CPU_hilbert, loadmat
+from AOT_biomaps.AOT_Acoustic.AcousticTools import calculate_envelope_squared, loadmat
 from .AcousticTools import next_power_of_2, reshape_field
 from .AcousticEnums import TypeSim, Dim, FormatSave, WaveType
 
@@ -19,10 +19,12 @@ from kwave.kspaceFirstOrder3D import kspaceFirstOrder3D
 from kwave.kspaceFirstOrder2D import kspaceFirstOrder2D
 from kwave.options.simulation_options import SimulationOptions
 from kwave.options.simulation_execution_options import SimulationExecutionOptions
+from AOT_biomaps.Settings import Params
 
 from tempfile import gettempdir
 from math import ceil
 from abc import ABC, abstractmethod
+import logging
 
 
 
@@ -75,7 +77,7 @@ class AcousticField(ABC):
             print(f"Initialization error: {e}")
             raise
         if params != None:
-            if type(params) != AOT_biomaps.Settings.Params:
+            if type(params) != Params:
                 raise TypeError("params must be an instance of the Params class")
 
             self.params = {
@@ -225,6 +227,7 @@ class AcousticField(ABC):
         Generate the acoustic field based on the specified simulation type and parameters.
         """
         try:
+            logging.getLogger('root').setLevel(logging.ERROR)
             if self.params['typeSim'] == TypeSim.FIELD2.value:
                 raise NotImplementedError("FIELD2 simulation is not implemented yet.")
             elif self.params['typeSim'] == TypeSim.KWAVE.value:
@@ -233,10 +236,10 @@ class AcousticField(ABC):
                         field = self._generate_acoustic_field_KWAVE_2D(isGpu, show_log)
                     except Exception as e:
                         raise RuntimeError(f"Failed to generate 2D acoustic field: {e}")
-                    self.field = calculate_envelope_squared(field, isGpu)
+                    self.field = reshape_field(calculate_envelope_squared(field),[self.factorT, self.factorX, self.factorZ])
                 elif self.params["dim"] == Dim.D3.value:
                     field = self._generate_acoustic_field_KWAVE_3D(isGpu, show_log)
-                    self.field = calculate_envelope_squared(field, isGpu)
+                    self.field = reshape_field(calculate_envelope_squared(field),[self.factorT, self.factorX, self.factorZ])
             elif self.params['typeSim'] == TypeSim.HYDRO.value:
                 raise ValueError("Cannot generate field for Hydrophone simulation, load exciting acquisitions.")
             else:
@@ -265,7 +268,7 @@ class AcousticField(ABC):
             print(f"Error in save_field method: {e}")
             raise
 
-    def load_field(self, folderPath, formatSave=FormatSave.HDR_IMG):
+    def load_field(self, folderPath, formatSave=FormatSave.HDR_IMG, nameBlock=None):
         """
         Load the acoustic field from a file in the specified format.
 
@@ -286,7 +289,7 @@ class AcousticField(ABC):
                         raise NotImplementedError("3D KWAVE field loading is not implemented yet.")
                 elif formatSave.value == FormatSave.H5.value:
                     if self.params["dim"] == Dim.D2.value:
-                         self._load_field_h5(folderPath)
+                         self._load_field_h5(folderPath,nameBlock)
                     elif self.params["dim"] == Dim.D3.value:
                         raise NotImplementedError("H5 KWAVE field loading is not implemented yet.")
                 elif formatSave.value == FormatSave.NPY.value:
@@ -502,7 +505,7 @@ class AcousticField(ABC):
         try:
             # --- 1. Grid setup ---
             dx = self.params['dx']
-            if dx >= self.params['element_width']:
+            if dx >= self.params['element_width']*2:
                 dx = self.params['element_width'] / 2
                 Nx = int(round((self.params['Xrange'][1] - self.params['Xrange'][0]) / dx))
                 Nz = int(round((self.params['Zrange'][1] - self.params['Zrange'][0]) / dx))
@@ -511,9 +514,9 @@ class AcousticField(ABC):
                 Nz = self.params['Nz']
 
             # --- 2. Time and space factors ---
-            factorT = int(np.ceil(self.params['f_AQ'] / self.params['f_saving']))
-            factorX = int(np.ceil(Nx / self.params['Nx']))
-            factorZ = int(np.ceil(Nz / self.params['Nz']))
+            self.factorT = int(np.ceil(self.params['f_AQ'] / self.params['f_saving']))
+            self.factorX = int(np.ceil(Nx / self.params['Nx']))
+            self.factorZ = int(np.ceil(Nz / self.params['Nz']))
 
             # --- 3. Grid and source initialization ---
             kgrid = kWaveGrid([Nx, Nz], [dx, dx])
@@ -531,6 +534,8 @@ class AcousticField(ABC):
             total_size_z = next_power_of_2(Nz)
             pml_x_size = (total_size_x - Nx) // 2
             pml_z_size = (total_size_z - Nz) // 2
+            pml_x_size = max(pml_x_size, 50)    # Ensure a minimum PML size of 50 grid points to avoid parasitic reflections
+            pml_z_size = max(pml_z_size, 50)    # Ensure a minimum PML size of 50 grid points to avoid parasitic reflections
 
             # --- 6. Simulation options ---
             simulation_options = SimulationOptions(
@@ -549,7 +554,7 @@ class AcousticField(ABC):
             )
 
             # --- 7. Call specialized function to set up source.p_mask and source.p ---
-            self._SetUpSource(source, Nx, dx, factorT)
+            self._SetUpSource(source, Nx, dx, self.factorT)
 
             # --- 8. Run simulation ---
             sensor_data = kspaceFirstOrder2D(
@@ -563,10 +568,7 @@ class AcousticField(ABC):
 
             # --- 9. Post-process results ---
             data = sensor_data['p'].reshape(kgrid.Nt, Nz, Nx)
-            if factorT != 1 or factorX != 1 or factorZ != 1:
-                return reshape_field(data, [factorT, factorX, factorZ])
-            else:
-                return data
+            return data
             
         except Exception as e:
             print(f"Error generating 2D acoustic field: {e}")
@@ -653,7 +655,7 @@ class AcousticField(ABC):
         """
         pass
 
-    def _load_field_h5(self, filePath):
+    def _load_field_h5(self, filePath,nameBlock):
         """
         Load the 2D acoustic field from an H5 file.
 
@@ -664,8 +666,10 @@ class AcousticField(ABC):
         - field (numpy.ndarray): The loaded acoustic field.
         """
         try:
-            with h5py.File(filePath+self.getName_field()+".h5", 'r') as f:
-                self.field = f['data'][:]
+            if nameBlock is None:
+                nameBlock = 'data'
+            with h5py.File(os.path.join(filePath, self.getName_field()+".h5"), 'r') as f:
+                self.field = f[nameBlock][:]
         except Exception as e:
             print(f"Error in _load_field_h5 method: {e}")
             raise
