@@ -47,7 +47,7 @@ class Tomography(Experiment):
             return False, f"OpticImage phantom shape {self.OpticImage.phantom.shape} does not match AcousticFields shape {self.AcousticFields[0].field.shape[1:]}."
         return True, "Experiment is correctly initialized."
 
-    def generateAcousticFields(self, fieldDataPath=None, show_log=True):
+    def generateAcousticFields(self, fieldDataPath=None, show_log=True, nameBlock=None):
         """
         Generate the acoustic fields for simulation.
         Args:
@@ -57,7 +57,7 @@ class Tomography(Experiment):
             systemMatrix: A numpy array of the generated fields.
         """
         if self.TypeAcoustic.value == WaveType.StructuredWave.value:
-            self.AcousticFields = self._generateAcousticFields_STRUCT_CPU(fieldDataPath, show_log)
+            self.AcousticFields = self._generateAcousticFields_STRUCT_CPU(fieldDataPath, show_log, nameBlock)
         else:
             raise ValueError("Unsupported wave type.")
 
@@ -389,7 +389,7 @@ class Tomography(Experiment):
         return True
 
     # PRIVATE METHODS
-    def _generateAcousticFields_STRUCT_CPU(self, fieldDataPath=None, show_log=False):
+    def _generateAcousticFields_STRUCT_CPU(self, fieldDataPath=None, show_log=False, nameBlock=None):
         if self.patterns is None:
             raise ValueError("patterns is not initialized. Please load or generate the active list first.")
         listAcousticFields = []
@@ -415,7 +415,7 @@ class Tomography(Experiment):
             if pathField is not None and os.path.exists(pathField):
                 progress_bar.set_postfix_str(f"Loading field - {AcousticField.getName_field()} -- Memory used: {memory.percent}%")
                 try:
-                    AcousticField.load_field(fieldDataPath, self.FormatSave)
+                    AcousticField.load_field(fieldDataPath, self.FormatSave,nameBlock)
                 except:
                     progress_bar.set_postfix_str(f"Error loading field -> Generating field - {AcousticField.getName_field()} -- Memory used: {memory.percent}% ---- processing on {config.get_process().upper()} ----")
                     AcousticField.generate_field(show_log=show_log)
