@@ -56,6 +56,7 @@ class Experiment(ABC):
         pass
 
     def cutAcousticFields(self, max_t, min_t=0):
+
         max_t = float(max_t)
         min_t = float(min_t)
 
@@ -76,6 +77,7 @@ class Experiment(ABC):
                 raise ValueError(f"Field {field.getName_field()} has an invalid shape: {field.field.shape}. Expected shape to be at least ({max_sample},).")
             self.AcousticFields[i].field = field.field[min_sample:max_sample, :, :]
 
+
     def addNoise(self, noiseType='gaussian', noiseLvl=0.1, withTumor=True):
         """
         Ajoute du bruit (gaussien ou poisson) au signal AO sélectionné.
@@ -111,11 +113,7 @@ class Experiment(ABC):
                 raise ValueError("noiseType must be either 'gaussian' or 'poisson'.")
             noisy_signal = np.clip(noisy_signal, a_min=0, a_max=None)  # Assurer que le signal reste non négatif
             noiseSignals[:, i] = noisy_signal
-
-        if withTumor:
-            self.AOsignal_withTumor = noiseSignals
-        else:
-            self.AOsignal_withoutTumor = noiseSignals
+        return noiseSignals
 
     def reduceDims(self, mode='avg'):
         """
@@ -157,6 +155,16 @@ class Experiment(ABC):
         for param in ['dx', 'dy', 'dz']:
             convert_and_update(self.params.general, param, lambda x: x * 2)
 
+    def normalizeAOsignals(self, withTumor=True):
+        if withTumor and self.AOsignal_withTumor is None:
+            raise ValueError("AO signal with tumor is not generated. Please generate it first.")
+        if not withTumor and self.AOsignal_withoutTumor is None:
+            raise ValueError("AO signal without tumor is not generated. Please generate it first.")
+        if withTumor:
+            self.AOsignal_withTumor = self.AOsignal_withTumor - np.min(self.AOsignal_withTumor)/(np.max(self.AOsignal_withTumor)-np.min(self.AOsignal_withTumor))
+        else:
+            self.AOsignal_withoutTumor = self.AOsignal_withoutTumor - np.min(self.AOsignal_withoutTumor)/(np.max(self.AOsignal_withoutTumor)-np.min(self.AOsignal_withoutTumor))
+
     def saveAcousticFields(self, save_directory):
         progress_bar = trange(len(self.AcousticFields), desc="Saving Acoustic Fields")
         for i in progress_bar:
@@ -233,67 +241,83 @@ class Experiment(ABC):
         return ani
 
     def generateAOsignal(self, withTumor=True, AOsignalDataPath=None):
-        if self.AcousticFields is None:
-            raise ValueError("AcousticFields is not initialized. Please generate the system matrix first.")
-
-        if self.OpticImage is None:
-            raise ValueError("OpticImage is not initialized. Please generate the phantom first.")
 
         if AOsignalDataPath is not None:
             if not os.path.exists(AOsignalDataPath):
                 raise FileNotFoundError(f"AO file {AOsignalDataPath} not found.")
-            AOmatrix = self._load_AOSignal(AOsignalDataPath)
-            if AOmatrix.shape[0] != self.AcousticFields[0].field.shape[0]:
-                print(f"AO signal shape {AOmatrix.shape} does not match the expected shape {self.AcousticFields[0].field.shape}. Generating corrected AO signal to match...")
+            if withTumor:
+                self.AOsignal_withTumor = self._loadAOSignal(AOsignalDataPath)
+                if self.AOsignal_withTumor.shape[0] != self.AcousticFields[0].field.shape[0]:
+                    print(f"AO signal shape {self.AOsignal_withTumor.shape} does not match the expected shape {self.AcousticFields[0].field.shape}. Resizing Acoustic fields...")
+                    self.cutAcousticFields(max_t=self.AOsignal_withTumor.shape[0] / float(self.params.acoustic['f_saving']), min_t=0)
             else:
-                return AOmatrix
+                self.AOsignal_withoutTumor = self._loadAOSignal(AOsignalDataPath)
+                if self.AOsignal_withoutTumor.shape[0] != self.AcousticFields[0].field.shape[0]:
+                    print(f"AO signal shape {self.AOsignal_withoutTumor.shape} does not match the expected shape {self.AcousticFields[0].field.shape}. Resizing Acoustic fields...")
+                    self.cutAcousticFields(max_t=self.AOsignal_withoutTumor.shape[0] / float(self.params.acoustic['f_saving']), min_t=0)
+        else:    
+            if self.AcousticFields is None:
+                raise ValueError("AcousticFields is not initialized. Please generate the system matrix first.")
 
-        if not all(field.field.shape == self.AcousticFields[0].field.shape for field in self.AcousticFields):
-            minShape = min([field.field.shape[0] for field in self.AcousticFields])
-            self.cutAcousticFields(minShape * self.params['fs_aq'])
-        else:
-            shape_field = self.AcousticFields[0].field.shape
+            if self.OpticImage is None:
+                raise ValueError("OpticImage is not initialized. Please generate the phantom first.")
+            
+            if not all(field.field.shape == self.AcousticFields[0].field.shape for field in self.AcousticFields):
+                minShape = min([field.field.shape[0] for field in self.AcousticFields])
+                self.cutAcousticFields(max_t=minShape * self.params['fs_aq'])
+            else:
+                shape_field = self.AcousticFields[0].field.shape
 
-        AOsignal = np.zeros((shape_field[0], len(self.AcousticFields)), dtype=np.float32)
+            AOsignal = np.zeros((shape_field[0], len(self.AcousticFields)), dtype=np.float32)
 
-        if withTumor:
-            description = "Generating AO Signal with Tumor"
-        else:
-            description = "Generating AO Signal without Tumor"
+            if withTumor:
+                description = "Generating AO Signal with Tumor"
+            else:
+                description = "Generating AO Signal without Tumor"
 
-        for i in trange(len(self.AcousticFields), desc=description):
-            for t in range(self.AcousticFields[i].field.shape[0]):
-                if withTumor:
-                    interaction = self.OpticImage.phantom * self.AcousticFields[i].field[t, :, :]
-                else:
-                    interaction = self.OpticImage.laser.intensity * self.AcousticFields[i].field[t, :, :]
-                AOsignal[t, i] = np.sum(interaction)
+            for i in trange(len(self.AcousticFields), desc=description):
+                for t in range(self.AcousticFields[i].field.shape[0]):
+                    if withTumor:
+                        interaction = self.OpticImage.phantom * self.AcousticFields[i].field[t, :, :]
+                    else:
+                        interaction = self.OpticImage.laser.intensity * self.AcousticFields[i].field[t, :, :]
+                    AOsignal[t, i] = np.sum(interaction)
 
-        if withTumor:
-            self.AOsignal_withTumor = AOsignal
-        else:
-            self.AOsignal_withoutTumor = AOsignal
+            if withTumor:
+                self.AOsignal_withTumor = AOsignal
+            else:
+                self.AOsignal_withoutTumor = AOsignal
 
     @staticmethod
-    def _loadAOSignal(cdh_file):
-        with open(cdh_file, "r") as file:
-            cdh_content = file.readlines()
+    def _loadAOSignal(AOsignalPath):
+        if AOsignalPath.endswith(".cdh"):
+            with open(AOsignalPath, "r") as file:
+                cdh_content = file.readlines()
 
-        n_events = int([line.split(":")[1].strip() for line in cdh_content if "Number of events" in line][0])
-        n_acquisitions = int([line.split(":")[1].strip() for line in cdh_content if "Number of acquisitions per event" in line][0])
+            # Extraction des paramètres
+            n_events = int([line.split(":")[1].strip() for line in cdh_content if "Number of events" in line][0])
+            n_acquisitions = int([line.split(":")[1].strip() for line in cdh_content if "Number of acquisitions per event" in line][0])
+            num_elements = int([line.split(":")[1].strip() for line in cdh_content if "Number of US transducers" in line][0])
 
-        AOsignal_matrix = np.zeros((n_events, n_acquisitions), dtype=np.float32)
+            # Initialisation avec la bonne forme : (n_acquisitions, n_events)
+            AOsignal_matrix = np.zeros((n_acquisitions, n_events), dtype=np.float32)
 
-        with open(cdh_file.replace(".cdh", ".cdf"), "rb") as file:
-            for event in range(n_events):
-                num_elements = int([line.split(":")[1].strip() for line in cdh_content if "Number of US transducers" in line][0])
+            with open(AOsignalPath.replace(".cdh", ".cdf"), "rb") as file:
                 hex_length = (num_elements + 3) // 4
-                file.read(hex_length // 2)
+                for event in range(n_events):
+                    file.read(hex_length // 2)  # Sauter l'en-tête de l'événement
+                    signal = np.frombuffer(file.read(n_acquisitions * 4), dtype=np.float32)
+                    AOsignal_matrix[:, event] = signal  # Remplir par colonne
 
-                signal = np.frombuffer(file.read(n_acquisitions * 4), dtype=np.float32)
-                AOsignal_matrix[event, :] = signal
+            AOsignal_matrix = AOsignal_matrix
+
+            return AOsignal_matrix
+
+        elif AOsignalPath.endswith(".npy"):
+            return np.load(AOsignalPath)  # Supposé déjà au bon format
+        else:
+            raise ValueError("Format de fichier non supporté. Utilisez .cdh/.cdf ou .npy.")
 
-        return AOsignal_matrix
 
     def saveAOsignals_Castor(self, save_directory, withTumor=True):
         if withTumor:
@@ -319,9 +343,9 @@ class Experiment(ABC):
                 fileID.write(AO_signal[:, j].astype(np.float32).tobytes())
 
         header_content = (
-            f"Data filename: AOSignals.cdf\n"
+            f"Data filename: {'AOSignals_withTumor.cdf' if withTumor else 'AOSignals_withoutTumor.cdf'}\n"
             f"Number of events: {nScan}\n"
-            f"Number of acquisitions per event: {AO_signal.shape[1]}\n"
+            f"Number of acquisitions per event: {AO_signal.shape[0]}\n"
             f"Start time (s): 0\n"
             f"Duration (s): 1\n"
             f"Acquisition frequency (Hz): {1/self.AcousticFields[0].kgrid.dt}\n"
@@ -475,18 +499,15 @@ class Experiment(ABC):
         plt.close(fig)
         return ani
 
-    def showPhantom(self):
+    def showPhantom(self, withROI=False):
         """
         Displays the optical phantom with absorbers.
         """
         try:
-            plt.imshow(self.OpticImage.phantom, extent=(self.OpticImage.laser.x[0], self.OpticImage.laser.x[-1] + 1, self.OpticImage.laser.z[-1], self.OpticImage.laser.z[0]), aspect='equal', cmap='hot')
-            plt.colorbar(label='Intensity')
-            plt.xlabel('X (mm)', fontsize=20)
-            plt.ylabel('Z (mm)', fontsize=20)
-            plt.tick_params(axis='both', which='major', labelsize=20)
-            plt.title('Optical Phantom with Absorbers')
-            plt.show()
+            if withROI:
+                self.OpticImage.show_ROI()
+            else:
+                self.OpticImage.show_phantom()
         except Exception as e:
             raise RuntimeError(f"Error plotting phantom: {e}")
         
