@@ -1,10 +1,9 @@
 from ._mainRecon import Recon
 from .ReconEnums import ReconType, OptimizerType, ProcessType
 from .AOT_Optimizers import MLEM, LS
-from .ReconTools import check_gpu_memory, calculate_memory_requirement, mse
+from .ReconTools import check_gpu_memory, calculate_memory_requirement, mse, load_recon
 from AOT_biomaps.Config import config
 
-
 import os
 import sys
 import subprocess
@@ -15,25 +14,25 @@ import matplotlib.animation as animation
 from IPython.display import HTML
 from datetime import datetime
 from tempfile import gettempdir
-
-
+import re
 
 class AlgebraicRecon(Recon):
     """
     This class implements the Algebraic reconstruction process.
     It currently does not perform any operations but serves as a template for future implementations.
     """
-    def __init__(self, opti = OptimizerType.MLEM, numIterations = 10000, numSubsets = 1, isSavingEachIteration=True, lambda_reg=None, L_Factor=None, **kwargs):
+    def __init__(self, opti = OptimizerType.MLEM, numIterations = 10000, numSubsets = 1, isSavingEachIteration=True, maxSaves = 5000, alpha = None, **kwargs):
         super().__init__(**kwargs)
         self.reconType = ReconType.Algebraic
         self.optimizer = opti
         self.reconPhantom = []
         self.reconLaser = []
+        self.indices = []
         self.numIterations = numIterations
         self.numSubsets = numSubsets
         self.isSavingEachIteration = isSavingEachIteration
-        self.lambda_reg = lambda_reg
-        self.L_Factor = L_Factor
+        self.maxSaves = maxSaves
+        self.alpha = alpha  # Regularization parameter for LS
 
         if self.numIterations <= 0:
             raise ValueError("Number of iterations must be greater than 0.")
@@ -62,19 +61,124 @@ class AlgebraicRecon(Recon):
         else:
             raise ValueError(f"Unknown Algebraic reconstruction type: {processType}")
 
-    def load_reconCASToR(self,withTumor = True):
-        if withTumor:
-            folder = 'results_withTumor'
-        else:
-            folder = 'results_withoutTumor'
-            
-        for thetaFiles in os.path.join(self.saveDir, folder + '_{}'):
-            if thetaFiles.endswith('.hdr'):
-                theta = Recon.load_recon(thetaFiles)
+    def load_reconCASToR(self, withTumor=True):
+        # Détermine le dossier et le préfixe des fichiers
+        folder = 'results_withTumor' if withTumor else 'results_withoutTumor'
+        folder_path = os.path.join(self.saveDir, folder)
+
+        # Liste tous les fichiers .img dans le dossier
+        img_files = [
+            f for f in os.listdir(folder_path)
+            if f.endswith('.img') and f.startswith(folder)
+        ]
+
+        # Fonction pour extraire le numéro d'itération (ex: "it56" → 56)
+        def get_iteration(filename):
+            match = re.search(r'_it(\d+)\.img$', filename)
+            return int(match.group(1)) if match else float('inf')  # Retourne l'infini pour les fichiers invalides (ils seront à la fin)
+
+        # Trie les fichiers par numéro d'itération (croissant)
+        sorted_files = sorted(img_files, key=get_iteration)
+
+        # Charge les données et remplit self.reconPhantom/self.reconLaser + self.indices
+        for file in sorted_files:
+            # Chemin complet du fichier .hdr correspondant
+            hdr_path = os.path.join(folder_path, file.replace('.img', '.hdr'))
+
+            # Vérifie que le .hdr existe avant de charger
+            if os.path.exists(hdr_path):
+                theta = load_recon(hdr_path)
+                iteration = get_iteration(file)
+
+                if iteration != float('inf'):  # Ignore les fichiers mal formatés
+                    if withTumor:
+                        self.reconPhantom.append(theta)
+                    else:
+                        self.reconLaser.append(theta)
+                    self.indices.append(iteration)
+
+    def load_reconPython(self, withTumor=True, results_date=None, optimizer=None, filePath=None):
+        if filePath is not None:
+            # Mode chargement direct depuis un fichier
+            recon_key = 'reconPhantom' if withTumor else 'reconLaser'
+            recon_path = filePath
+            if not os.path.exists(recon_path):
+                raise FileNotFoundError(f"No reconstruction file found at {recon_path}.")
+            # Charge les données
+            data = np.load(recon_path, allow_pickle=True)
+            # Découpe en liste de 2D si c'est un tableau 3D
+            if isinstance(data, np.ndarray) and data.ndim == 3:
                 if withTumor:
-                    self.reconPhantom.append(theta)
+                    self.reconPhantom = [data[i, :, :] for i in range(data.shape[0])]
                 else:
-                    self.reconLaser.append(theta)
+                    self.reconLaser = [data[i, :, :] for i in range(data.shape[0])]
+            else:
+                # Si ce n'est pas un tableau 3D, on suppose que c'est déjà une liste de 2D
+                if withTumor:
+                    self.reconPhantom = data
+                else:
+                    self.reconLaser = data
+            # Essayer de charger les indices
+            base_dir, file_name = os.path.split(recon_path)
+            file_base, _ = os.path.splitext(file_name)
+            indices_path = os.path.join(base_dir, f"indices.npy")
+            if os.path.exists(indices_path):
+                indices_data = np.load(indices_path, allow_pickle=True)
+                if isinstance(indices_data, np.ndarray) and indices_data.ndim == 3:
+                    self.indices = [indices_data[i, :, :] for i in range(indices_data.shape[0])]
+                else:
+                    self.indices = indices_data
+            else:
+                self.indices = None
+            print(f"Loaded reconstruction results and indices from {recon_path}")
+        else:
+            # Mode chargement depuis le répertoire de résultats
+            if self.saveDir is None:
+                raise ValueError("Save directory is not specified. Please set saveDir before loading.")
+            # Determine optimizer name for path matching
+            opt_name = optimizer.value if optimizer is not None else self.optimizer.value
+            # Find the most recent results directory if no date is specified
+            if results_date is None:
+                dirs = [
+                    d for d in os.listdir(self.saveDir)
+                    if os.path.isdir(os.path.join(self.saveDir, d))
+                    and re.match(r'results_\d{4}_' + re.escape(opt_name) + r'($|_)', d)
+                ]
+                if not dirs:
+                    raise FileNotFoundError(f"No results directory found for optimizer '{opt_name}' in {self.saveDir}.")
+                dirs.sort(reverse=True)  # Most recent first
+                results_dir = os.path.join(self.saveDir, dirs[0])
+            else:
+                results_dir = os.path.join(self.saveDir, f'results_{results_date}_{opt_name}')
+                if not os.path.exists(results_dir):
+                    raise FileNotFoundError(f"Directory {results_dir} does not exist.")
+            # Load reconstruction results
+            recon_key = 'reconPhantom' if withTumor else 'reconLaser'
+            recon_path = os.path.join(results_dir, f'{recon_key}.npy')
+            if not os.path.exists(recon_path):
+                raise FileNotFoundError(f"No reconstruction file found at {recon_path}.")
+            data = np.load(recon_path, allow_pickle=True)
+            # Découpe en liste de 2D si c'est un tableau 3D
+            if isinstance(data, np.ndarray) and data.ndim == 3:
+                if withTumor:
+                    self.reconPhantom = [data[i, :, :] for i in range(data.shape[0])]
+                else:
+                    self.reconLaser = [data[i, :, :] for i in range(data.shape[0])]
+            else:
+                if withTumor:
+                    self.reconPhantom = data
+                else:
+                    self.reconLaser = data
+            # Try to load saved indices (if file exists)
+            indices_path = os.path.join(results_dir, 'indices.npy')
+            if os.path.exists(indices_path):
+                indices_data = np.load(indices_path, allow_pickle=True)
+                if isinstance(indices_data, np.ndarray) and indices_data.ndim == 3:
+                    self.indices = [indices_data[i, :, :] for i in range(indices_data.shape[0])]
+                else:
+                    self.indices = indices_data
+            else:
+                self.indices = None
 
     def plot_MSE(self, isSaving=True, log_scale_x=False, log_scale_y=False):
         """
@@ -119,12 +223,7 @@ class AlgebraicRecon(Recon):
                 scale_str = "_logx"
             elif log_scale_y:
                 scale_str = "_logy"
-            if self.optimizer == OptimizerType.MLEM:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_MLEM{scale_str}{date_str}.png')
-            elif self.optimizer == OptimizerType.LS:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS{scale_str}{date_str}.png')
-            elif self.optimizer == OptimizerType.LS_TV:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}{scale_str}{date_str}.png')
+            SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_{self.optimizer.name}_{scale_str}{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"MSE plot saved to {SavingFolder}")
 
@@ -192,13 +291,7 @@ class AlgebraicRecon(Recon):
             savePath = os.path.join(self.saveDir, 'results')
             if not os.path.exists(savePath):
                 os.makedirs(savePath)
-            if self.optimizer == OptimizerType.MLEM:
-                namePath = f'{self.SMatrix.shape[3]}_SCANS_comparison_MSE_BestANDLastRecon_MLEM_Date_{date_str}.png'
-            elif self.optimizer == OptimizerType.LS:
-                namePath = f'{self.SMatrix.shape[3]}_SCANS_comparison_MSE_BestANDLastRecon_LS_Date_{date_str}.png'
-            elif self.optimizer == OptimizerType.LS_TV:
-                namePath = f'{self.SMatrix.shape[3]}_SCANS_comparison_MSE_BestANDLastRecon_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_Date_{date_str}.png'
-            SavingFolder = os.path.join(savePath, namePath)
+            SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_comparison_MSE_BestANDLastRecon_{self.optimizer.name}_{date_str}.png')
             plt.savefig(SavingFolder, dpi=300, bbox_inches='tight')
             print(f"MSE plot saved to {SavingFolder}")
 
@@ -345,12 +438,7 @@ class AlgebraicRecon(Recon):
                 scale_str = "_logx"
             elif log_scale_y:
                 scale_str = "_logy"
-            if self.optimizer == OptimizerType.MLEM:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_MLEM{scale_str}{date_str}.png')
-            elif self.optimizer == OptimizerType.LS:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS{scale_str}{date_str}.png')
-            elif self.optimizer == OptimizerType.LS_TV:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}{scale_str}{date_str}.png')
+            SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_{self.optimizer.name}_{scale_str}{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"SSIM plot saved to {SavingFolder}")
 
@@ -367,9 +455,6 @@ class AlgebraicRecon(Recon):
         # ----------------- Plotting -----------------
         _, axs = plt.subplots(1, 3, figsize=(15, 5))  # 1 row, 3 columns
 
-        # Normalization based on LAMBDA max
-        lambda_max = np.max(self.experiment.OpticImage.laser.intensity)
-
         # Left: Best reconstructed image (normalized)
         im0 = axs[0].imshow(best_recon, 
                             extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
@@ -402,18 +487,18 @@ class AlgebraicRecon(Recon):
         if isSaving:
             now = datetime.now()    
             date_str = now.strftime("%Y_%d_%m_%y")
-            SavingFolder = os.path.join(self.saveDir, 'results', f'comparison_SSIM_BestANDLastRecon{date_str}.png')
+            SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_comparison_SSIM_BestANDLastRecon_{self.optimizer.name}_{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"SSIM plot saved to {SavingFolder}")
         plt.show()
 
-    def plot_CRC_vs_Noise(self, ROI_mask = None, start=0, fin=None, step=10, save_path=None):
+    def plot_CRC_vs_Noise(self, use_ROI=True, fin=None, isSaving=True):
         """
         Plot CRC (Contrast Recovery Coefficient) vs Noise for each iteration.
         """
         if self.reconLaser is None or self.reconLaser == []:
             raise ValueError("Reconstructed laser is empty. Run reconstruction first.")
-        if isinstance(self.Laser,list) and len(self.Laser) == 1:
+        if isinstance(self.reconLaser, list) and len(self.reconLaser) == 1:
             raise ValueError("Reconstructed Image without tumor is a single frame. Run reconstruction with isSavingEachIteration=True to get a sequence of frames.")
         if self.reconPhantom is None or self.reconPhantom == []:
             raise ValueError("Reconstructed phantom is empty. Run reconstruction first.")
@@ -424,24 +509,21 @@ class AlgebraicRecon(Recon):
             fin = len(self.reconPhantom) - 1
 
         iter_range = self.indices
+        
+        if self.CRC is None:
+            self.calculateCRC(use_ROI=use_ROI)
 
-        crc_values = []
         noise_values = []
 
         for i in iter_range:
             recon_without_tumor = self.reconLaser[i].T
-
-            # CRC
-            crc = self.calculateCRC(iteration=i,ROI_mask=ROI_mask)
-            crc_values.append(crc)
-
             # Noise
             noise = np.mean(np.abs(recon_without_tumor - self.experiment.OpticImage.laser.intensity))
             noise_values.append(noise)
 
         plt.figure(figsize=(6, 5))
-        plt.plot(noise_values, crc_values, 'o-', label='ML-EM')
-        for i, (x, y) in zip(iter_range, zip(noise_values, crc_values)):
+        plt.plot(noise_values, self.CRC, 'o-', label=self.optimizer.name)
+        for i, (x, y) in zip(iter_range, zip(noise_values, self.CRC)):
             plt.text(x, y, str(i), fontsize=5.5, ha='left', va='bottom')
 
         plt.xlabel("Noise (mean absolute error)")
@@ -453,10 +535,12 @@ class AlgebraicRecon(Recon):
         plt.title("CRC vs Noise over Iterations")
         plt.grid(True)
         plt.legend()
-
-        if save_path:
-            plt.savefig(save_path, dpi=300)
-            print(f"Figure saved to: {save_path}")
+        if isSaving:
+            now = datetime.now()    
+            date_str = now.strftime("%Y_%d_%m_%y")
+            SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_CRCvsNOISE_{self.optimizer.name}_{date_str}.png')
+            plt.savefig(SavingFolder, dpi=300)
+            print(f"CRCvsNOISE plot saved to {SavingFolder}")
         plt.show()
         
     def show_reconstruction_progress(self, start=0, fin=None, save_path=None, with_tumor=True):
@@ -568,46 +652,43 @@ class AlgebraicRecon(Recon):
 
         plt.show()
 
-    def save(self, withTumor=True):
+    def checkExistingFile(self, date = None):
         """
-        Save the reconstruction results (reconPhantom is with tumor, reconLaser is without tumor) and indices of the saved recon results, in format numpy.
-        Warnings : reconPhantom and reconLaser are lists of 2D numpy arrays, each array corresponding to one iteration.
-        """
-        if self.saveDir is None:
-            raise ValueError("Save directory is not specified. Please set saveDir before saving.")
-        if withTumor:
-            if not self.reconPhantom or len(self.reconPhantom) == 0:
-                raise ValueError("Reconstructed phantom is empty. Run reconstruction first.")
-            np.save(os.path.join(self.saveDir, 'results', 'reconPhantom.npy'), np.array(self.reconPhantom))
-        else:
-            if not self.reconLaser or len(self.reconLaser) == 0:
-                raise ValueError("Reconstructed laser is empty. Run reconstruction first.")
-            np.save(os.path.join(self.saveDir, 'results', 'reconLaser.npy'), np.array(self.reconLaser))
-        np.save(os.path.join(self.saveDir, 'results', 'reconIndices.npy'), np.array(self.indices))
-        print(f"Reconstruction results saved to {os.path.join(self.saveDir, 'results')}")
+        Check if the reconstruction file already exists, based on current instance parameters.
 
-    def load(self, withTumor=True):
-        """
-        Load the reconstruction results (reconPhantom is with tumor, reconLaser is without tumor) and indices of the saved recon results, in format numpy.
-        Warnings : reconPhantom and reconLaser are lists of 2D numpy arrays, each array corresponding to one iteration.
+        Args:
+            withTumor (bool): If True, checks reconPhantom.npy; otherwise, checks reconLaser.npy.
+            overwrite (bool): If False, returns False if the file exists.
+
+        Returns:
+            tuple: (bool: whether to save, str: the filepath)
         """
         if self.saveDir is None:
-            raise ValueError("Save directory is not specified. Please set saveDir before loading.")
-        if withTumor:
-            recon_path = os.path.join(self.saveDir, 'results', 'reconPhantom.npy')
-            if not os.path.isfile(recon_path):
-                raise FileNotFoundError(f"Reconstructed phantom file not found: {recon_path}")
-            self.reconPhantom = list(np.load(recon_path, allow_pickle=True))
-        else:
-            recon_path = os.path.join(self.saveDir, 'results', 'reconLaser.npy')
-            if not os.path.isfile(recon_path):
-                raise FileNotFoundError(f"Reconstructed laser file not found: {recon_path}")
-            self.reconLaser = list(np.load(recon_path, allow_pickle=True))
-        indices_path = os.path.join(self.saveDir, 'results', 'reconIndices.npy')
-        if not os.path.isfile(indices_path):
-            raise FileNotFoundError(f"Reconstruction indices file not found: {indices_path}")
-        self.indices = list(np.load(indices_path, allow_pickle=True))
-        print(f"Reconstruction results loaded from {os.path.join(self.saveDir, 'results')}")
+            raise ValueError("Save directory is not specified.")
+        if date is None:
+            date = datetime.now().strftime("%d%m")
+        results_dir = os.path.join(self.saveDir, f'results_{date}_{self.optimizer.value}')
+        if not os.path.exists(results_dir):
+            os.makedirs(results_dir)
+
+        if os.path.exists(os.path.join(results_dir,"reconIndices.npy")):
+            return (True, results_dir)
+
+        return (False, results_dir)
+
+    def load(self, processType = ProcessType.PYTHON, withTumor=True, results_date=None, optimizer=None, filePath=None):
+        """
+        Load the reconstruction results (reconPhantom or reconLaser) and indices as lists of 2D np arrays into self.
+        If the loaded file is a 3D array, it is split into a list of 2D arrays.
+        Args:
+            withTumor: If True, loads reconPhantom (with tumor), else reconLaser (without tumor).
+            results_date: Date string (format "ddmm") to specify which results to load. If None, uses the most recent date in saveDir.
+            optimizer: Optimizer name (as string or enum) to filter results. If None, uses the current optimizer of the instance.
+            filePath: Optional. If provided, loads directly from this path (overrides saveDir and results_date).
+        """
+
+        pass
+        # print(f"Loaded reconstruction results and indices from {results_dir}")
 
     def normalizeSMatrix(self):
         self.SMatrix = self.SMatrix / (float(self.experiment.params.acoustic['voltage'])*float(self.experiment.params.acoustic['sensitivity']))  
@@ -619,59 +700,70 @@ class AlgebraicRecon(Recon):
         if withTumor:
             if self.experiment.AOsignal_withTumor is None:
                 raise ValueError("AO signal with tumor is not available. Please generate AO signal with tumor the experiment first in the experiment object.")
-            if self.optimizer.value == OptimizerType.MLEM.value:
-                self.reconPhantom, self.indices = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
-            elif self.optimizer.value == OptimizerType.LS.value:
-                self.reconPhantom, self.indices = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
-            elif self.optimizer.value == OptimizerType.LS_TV.value:
-                self.reconPhantom, self.indices = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withTumor, withTumor=withTumor)
             else:
-                raise ValueError(f"Only MLEM and LS are supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
+                y = self.experiment.AOsignal_withTumor
         else:
             if self.experiment.AOsignal_withoutTumor is None:
                 raise ValueError("AO signal without tumor is not available. Please generate AO signal without tumor the experiment first in the experiment object.")
-            if self.optimizer.value == OptimizerType.MLEM.value:
-                self.reconLaser, self.indices = self._MLEM(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
-            elif self.optimizer.value == OptimizerType.LS.value:
-                self.reconLaser, self.indices = self._LS(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
-            elif self.optimizer.value == OptimizerType.LS_TV.value:
-                self.reconLaser, self.indices = self._LS_Regularized(SMatrix=self.SMatrix, y=self.experiment.AOsignal_withoutTumor, withTumor=withTumor)
             else:
-                raise ValueError(f"Only MLEM and LS are supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
+                y = self.experiment.AOsignal_withoutTumor
+
+        if self.optimizer.value == OptimizerType.MLEM.value:
+            self.reconPhantom, self.indices = MLEM(SMatrix=self.SMatrix, 
+                                                    y=y,
+                                                    numIterations=self.numIterations,
+                                                    isSavingEachIteration=self.isSavingEachIteration,
+                                                    withTumor=withTumor,
+                                                    use_multi_gpu= self.isMultiGPU,
+                                                    use_numba= self.isMultiCPU,
+                                                    max_saves=self.maxSaves
+                                                    )
+        elif self.optimizer.value == OptimizerType.LS.value:
+            if self.alpha is None:
+                raise ValueError("Alpha (regularization parameter) must be set for LS reconstruction.")
+            self.reconPhantom, self.indices = LS(SMatrix=self.SMatrix,
+                                                y=y, 
+                                                numIterations=self.numIterations, 
+                                                isSavingEachIteration=self.isSavingEachIteration, 
+                                                withTumor=withTumor,
+                                                alpha=self.alpha,
+                                                max_saves=self.maxSaves,
+                                                )
+        else:
+            raise ValueError(f"Only MLEM and LS are supported for simple algebraic reconstruction. {self.optimizer.value} need Bayesian reconstruction")
 
     def _AlgebraicReconCASToR(self, withTumor):
-        
-        # Define variables
-        smatrix = os.path.join(self.saveDir,"system_matrix")
-
+        # Définir les chemins
+        smatrix = os.path.join(self.saveDir, "system_matrix")
         if withTumor:
             fileName = 'AOSignals_withTumor.cdh'
         else:
             fileName = 'AOSignals_withoutTumor.cdh'
 
-        # Check if input file exists
-        if not os.path.isfile(f"{self.saveDir}/{fileName}"):
+        # Vérifier et générer les fichiers d'entrée si nécessaire
+        if not os.path.isfile(os.path.join(self.saveDir, fileName)):
+            print(f"Fichier .cdh manquant. Génération de {fileName}...")
             self.experiment.saveAOsignals_Castor(self.saveDir)
-        # Check if system matrix directory exists
-        elif not os.path.isdir(smatrix):
-            os.mkdir(smatrix)
-        # check if system matrix is empty
-        elif not os.listdir(smatrix):
+
+        # Vérifier/générer la matrice système
+        if not os.path.isdir(smatrix):
+            os.makedirs(smatrix, exist_ok=True)
+        if not os.listdir(smatrix):
+            print("Matrice système manquante. Génération...")
             self.experiment.saveAcousticFields(self.saveDir)
 
-        # Vérifier que le dossier de sortie existe
-        os.makedirs(os.path.join(self.saveDir, 'results','recon'), exist_ok=True)
+        # Vérifier que le fichier .cdh existe (redondant mais sûr)
+        if not os.path.isfile(os.path.join(self.saveDir, fileName)):
+            raise FileNotFoundError(f"Le fichier .cdh n'existe toujours pas : {fileName}")
 
-        # Vérifier que le fichier .cdh existe
-        if not os.path.isfile(fileName):
-            raise FileNotFoundError(f"Le fichier .cdh n'existe pas : {fileName}")
+        # Créer le dossier de sortie
+        os.makedirs(os.path.join(self.saveDir, 'results', 'recon'), exist_ok=True)
 
-        # Créer une copie de l'environnement actuel et ajouter les variables CAStOR
+        # Configuration de l'environnement pour CASToR
         env = os.environ.copy()
-
         env.update({
-            "CASTOR_DIR": f"{self.experiment.params.reconstruction['castor_executable']}",
-            "CASTOR_CONFIG": f"{self.experiment.params.reconstruction['castor_executable']}/config",
+            "CASTOR_DIR": self.experiment.params.reconstruction['castor_executable'],
+            "CASTOR_CONFIG": os.path.join(self.experiment.params.reconstruction['castor_executable'], "config"),
             "CASTOR_64bits": "1",
             "CASTOR_OMP": "1",
             "CASTOR_SIMD": "1",
@@ -680,13 +772,13 @@ class AlgebraicRecon(Recon):
 
         # Construire la commande
         cmd = [
-            f"{self.experiment.params.reconstruction['castor_executable']}/bin/castor-recon",
-            "-df",  f"{self.saveDir}/{fileName}",
+            os.path.join(self.experiment.params.reconstruction['castor_executable'], "bin", "castor-recon"),
+            "-df", os.path.join(self.saveDir, fileName),
             "-opti", self.optimizer.value,
             "-it", f"{self.numIterations}:{self.numSubsets}",
             "-proj", "matrix",
-            "-dout", os.path.join(self.saveDir, 'results','recon'),
-            "-th", f"{ os.cpu_count()}",
+            "-dout", os.path.join(self.saveDir, 'results', 'recon'),
+            "-th", str(os.cpu_count()),
             "-vb", "5",
             "-proj-comp", "1",
             "-ignore-scanner",
@@ -695,90 +787,34 @@ class AlgebraicRecon(Recon):
             "-system-matrix", smatrix,
         ]
 
-        # Print the command
+        # Afficher la commande (pour débogage)
+        print("Commande CASToR :")
         print(" ".join(cmd))
 
-        #save the command to a script file
+        # Chemin du script temporaire
         recon_script_path = os.path.join(gettempdir(), 'recon.sh')
+
+        # Écrire le script bash
         with open(recon_script_path, 'w') as f:
             f.write("#!/bin/bash\n")
+            f.write(f"export PATH={env['CASTOR_DIR']}/bin:$PATH\n")  # Ajoute le chemin de CASToR au PATH
+            f.write(f"export LD_LIBRARY_PATH={env['CASTOR_DIR']}/lib:$LD_LIBRARY_PATH\n")  # Ajoute les bibliothèques si nécessaire
             f.write(" ".join(cmd) + "\n")
 
-        sys.exit(0)
-
-        # --- Run Reconstruction Script ---
-        print(f"Running reconstruction script: {recon_script_path}")
+        # Rendre le script exécutable et l'exécuter
         subprocess.run(["chmod", "+x", recon_script_path], check=True)
-        subprocess.run([recon_script_path], check=True)
-        print("Reconstruction script executed.")
+        print(f"Exécution de la reconstruction avec CASToR...")
+        result = subprocess.run(recon_script_path, env=env, check=True, capture_output=True, text=True)
 
+        # Afficher la sortie de CASToR (pour débogage)
+        print("Sortie CASToR :")
+        print(result.stdout)
+        if result.stderr:
+            print("Erreurs :")
+            print(result.stderr)
+
+        print("Reconstruction terminée avec succès.")
         self.load_reconCASToR(withTumor=withTumor)
-
-    def _MLEM(self, SMatrix, y, withTumor):
-        """
-        This method implements the MLEM algorithm using either CPU or single-GPU PyTorch acceleration.
-        Multi-GPU mode is disabled due to memory fragmentation issues or lack of availability.
-        """
-        result = None
-        indices = None
-        required_memory = calculate_memory_requirement(SMatrix, y)
-
-        if self.isGPU:
-            if check_gpu_memory(config.select_best_gpu(), required_memory):
-                result, indices = MLEM._MLEM_GPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-            else:
-                warnings.warn("Insufficient GPU memory for single GPU MLEM. Falling back to CPU.")
-
-        if result is None and self.isMultiCPU:
-            result, indices = MLEM._MLEM_CPU_multi(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-
-        if result is None:
-            result, indices = MLEM._MLEM_CPU_opti(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-            if result is None:
-                warnings.warn("Optimized MLEM failed. Falling back to basic CPU MLEM.")
-                result, indices = MLEM._MLEM_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-
-        return result, indices
-    
-    def _LS(self, SMatrix, y, withTumor):
-        """
-        This method implements the LS algorithm using either CPU or single-GPU PyTorch acceleration.
-        Multi-GPU mode is disabled due to memory fragmentation issues or lack of availability.
-        """
-        result = None
-        indices = None
-        required_memory = calculate_memory_requirement(SMatrix, y)
-
-        if self.isGPU:
-            if check_gpu_memory(config.select_best_gpu(), required_memory):
-                result, indices = LS._LS_GPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-            else:
-                warnings.warn("Insufficient GPU memory for single GPU LS. Falling back to CPU.")
-
-        if result is None and self.isMultiCPU:
-            result, indices = LS._LS_CPU_multi(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-
-        if result is None:
-            result, indices = LS._LS_CPU_opti(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-            if result is None:
-                warnings.warn("Optimized LS failed. Falling back to basic CPU LS.")
-                result, indices = LS._LS_CPU_basic(SMatrix=SMatrix, y=y, numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor)
-
-        return result, indices
-
-    def _LS_Regularized(self,SMatrix, y, withTumor):
-        result = None
-        indices = None
-        required_memory = calculate_memory_requirement(SMatrix, y)
-
-        if check_gpu_memory(config.select_best_gpu(), required_memory):
-            if self.lambda_reg is None or self.L_Factor is None:
-                raise ValueError("For LS with TV regularization, both lambda_reg and L_Factor must be specified.")
-            result, indices = LS._LS_TV_GPU(SMatrix= SMatrix, y=y,  numIterations=self.numIterations, isSavingEachIteration=self.isSavingEachIteration, withTumor=withTumor, lambda_tv=self.lambda_reg, L_Factor=self.L_Factor)
-        else:
-            warnings.warn("Insufficient GPU memory for single GPU LS with TV regularization. Falling back to CPU.")
-            raise NotImplementedError("LS with TV regularization is not implemented for CPU.")
-        return result, indices
     
     # STATIC METHODS
     @staticmethod
