@@ -29,4 +29,7 @@ class DeepLearningRecon(Recon):
         pass
 
     def _deepLearningReconPython(self):
-        pass
\ No newline at end of file
+        pass
+
+    def checkExistingFile(self, date = None):
+        raise NotImplementedError("checkExistingFile method is not implemented yet.")
\ No newline at end of file
